from pytz import timezone

from fild.sdk import Field, dates


class DbTimestamp(Field):
    def generate_value(self):
        return dates.generate_time()

    def to_db(self):
        return self.value

    def to_format(self, fmt=dates.Pattern.DATE):
        return self.value.strftime(fmt)

    def to_timezone(self, tz):
        return self.value.astimezone(tz=timezone(tz))



import json

from fild.sdk import Array, Bool, Dictionary


class DbBool(Bool):
    def to_db(self):
        return int(self.value)

    def with_values(self, values):
        if isinstance(values, int):
            values = bool(values)

        self._value = values

        return self


class DBJsonDict(Dictionary):
    def to_db(self):
        return json.dumps(self.value, separators=(',', ':'))

    def with_values(self, values):
        if isinstance(values, str):
            values = json.loads(values)

        if values is not None:
            return super().with_values(values)

        return self


class DbJsonArray(Array):
    def to_db(self):
        return json.dumps(self.value, separators=(',', ':'))

    def with_values(self, values):
        if isinstance(values, str):
            values = json.loads(values)

        if values is not None:
            return super().with_values(values)

        return self



from datetime import datetime

from fild_compare import compare
from pytz import timezone

from fild_db.types.common import DbTimestamp
from fild_db.types.mysql import DbBool, DbJsonArray
from tests.data import SampleDict

DATE_VALUE = datetime.now().replace(
    year=2022, month=5, day=13, hour=12, minute=0, second=10, microsecond=0
)
TZ = datetime.now().replace(
    year=2022, month=5, day=13, hour=12, minute=0, second=10, microsecond=0
).astimezone(tz=timezone('CET'))
DATE = DbTimestamp().with_values(DATE_VALUE)


def test_to_db():
    assert DATE.to_db() == DATE_VALUE


def test_to_timezone():
    assert DATE.to_timezone('CET') == TZ


def test_to_text():
    assert DATE.to_format() == '2022-05-13'


def test_mysql_bool_with_values():
    assert DbBool().with_values(1).value is True


def test_mysql_bool_to_db():
    assert DbBool().with_values(0).to_db() == 0


def test_json_dict_with_values():
    compare(
        actual=SampleDict().with_values('{"id":234,"name":"val"}').value,
        expected={
            SampleDict.Id.name: 234,
            SampleDict.Name.name: 'val'
        }
    )


def test_json_dict_to_db():
    assert SampleDict().with_values({
        SampleDict.Id.name: 1,
        SampleDict.Name.name: 'test'
    }).with_values(None).to_db() == '{"id":1,"name":"test"}'


def test_json_array_to_db():
    assert DbJsonArray(SampleDict).with_values([{
        SampleDict.Id.name: 1,
        SampleDict.Name.name: 'test'
    }]).with_values(None).to_db() == '[{"id":1,"name":"test"}]'


def test_json_array_with_values():
    compare(
        actual=DbJsonArray(SampleDict).with_values(
            '[{"id":234,"name":"val"}]'
        ).value,
        expected=[{
            SampleDict.Id.name: 234,
            SampleDict.Name.name: 'val'
        }]
    )



from fild_compare import compare
from fild_compare.rules import has_some_value

from tests.data import Model, Table


def test_model():
    assert Model.get_table_name() == 'table_name'


def test_regular_data():
    compare(
        actual=Model().to_db(),
        expected={
            'id': 1, 'is_global': True, 'metadata_column': 'm', 'name': 'name'
        },
        rules={
            'id': has_some_value,
            'is_global': has_some_value,
            'metadata_column': has_some_value,
            'name': has_some_value
        }
    )


def test_custom_data():
    compare(
        actual=Model(is_custom=True).to_db(),
        expected={'id': 1, 'name': 'name'},
        rules={
            'id': has_some_value,
            'name': has_some_value
        }
    )


def test_full_data():
    compare(
        actual=Model(is_full=True).to_db(),
        expected={
            'id': 1,
            'is_global': True,
            'metadata_column': 'm',
            'name': 'name',
            'comment': 's',
            'created_at': 'test'
        },
        rules={
            'id': has_some_value,
            'is_global': has_some_value,
            'metadata_column': has_some_value,
            'name': has_some_value,
            'comment': has_some_value,
            'created_at': has_some_value
        }
    )


def test_to_table_record():
    assert isinstance(Model().to_table_record(), Table)
